@@ -1,22 +1,23 @@
 import builtins
 import typing as ty
-
+import numbers
 from immutabledict import immutabledict
+import warnings
 
 import strax
+
 export, __all__ = strax.exporter()
 
 # Placeholder value for omitted values.
 # Use instead of None since None might be a proper value/default
 OMITTED = '<OMITTED>'
-__all__.append('OMITTED')
+__all__ += 'OMITTED InvalidConfiguration'.split()
 
 
 @export
 class InvalidConfiguration(Exception):
     pass
 
-
 @export
 def takes_config(*options):
     """Decorator for plugin classes, to specify which options it takes.
@@ -37,13 +38,15 @@ def takes_config(*options):
             # Already have some options set, e.g. because of subclassing
             # where both child and parent have a takes_config decorator
             for opt in result.values():
-                if opt.name in plugin_class.takes_config and not opt.overwrite:
+                if opt.name in plugin_class.takes_config:
                     raise RuntimeError(
                         f"Attempt to specify option {opt.name} twice")
             plugin_class.takes_config = immutabledict({
                 **plugin_class.takes_config, **result})
         else:
             plugin_class.takes_config = immutabledict(result)
+        if isinstance(opt, strax.Config):
+            setattr(plugin_class, opt.name, opt)
         return plugin_class
 
     return wrapped
@@ -56,14 +59,14 @@ class Option:
 
     def __init__(self,
                  name: str,
-                 type: type = OMITTED,
+                 type: ty.Union[type, tuple, list] = OMITTED,
                  default: ty.Any = OMITTED,
                  default_factory: ty.Callable = OMITTED,
                  default_by_run=OMITTED,
                  child_option: bool = False,
                  parent_option_name: str = None,
-                 overwrite: bool = False,
                  track: bool = True,
+                 infer_type = False,
                  help: str = ''):
         """
         :param name: Option identifier
@@ -74,17 +77,18 @@ class Option:
          - Callable. Will be called with run_id, must return value for run.
          - List [(start_run_id, value), ..,] for values specified by range of
            runs.
+         "default_by_run" can only be usd in contexts where the context option
+         "use_per_run_defaults" is set to True
         :param child_option: If true option is marked as a child_option. All
             options which are marked as a child overwrite the corresponding parent
             option. Removes also the corresponding parent option from the lineage.
         :param parent_option_name: Name of the parent option of child option.
             Required to find the key of the parent option so it can be overwritten
             by the value of the child option.
-        :param overwrite: allow this option to overwrite if there is
-            another option already registered with the same name with a
-            different value. Only use for subclassing plugins.
         :param track: If True (default), option value becomes part of plugin
         lineage (just like the plugin version).
+        :param infer_type: Whether to infer the type from the
+            default value if type not explicitly set.
         :param help: Human-readable description of the option.
         """
         self.name = name
@@ -92,7 +96,6 @@ class Option:
         self.default = default
         self.default_by_run = default_by_run
         self.default_factory = default_factory
-        self.overwrite = overwrite
         self.track = track
         self.help = help
 
@@ -113,15 +116,25 @@ class Option:
         #                   f" which will soon stop working!",
         #                   DeprecationWarning)
 
-        type = builtins.type
         if sum([self.default is not OMITTED,
                 self.default_factory is not OMITTED,
                 self.default_by_run is not OMITTED]) > 1:
             raise RuntimeError(f"Tried to specify more than one default "
                                f"for option {self.name}.")
-
-        if type is OMITTED and default is not OMITTED:
-            self.type = type(default)
+            
+        if infer_type and type is OMITTED and default is not OMITTED:
+            for ntype in [numbers.Integral, numbers.Number]:
+                # first check if its a number otherwise numpy numbers
+                # will fail type checking when checked against int and float.
+                # numbers.Integral, numbers.Number are safe to use
+                # since numpy registers them as super-classes.
+                # left as a loop since we may want to add other exceptions as
+                # they are discovered.
+                if isinstance(default, ntype):
+                    self.type = ntype
+                    break
+            else:
+                self.type = builtins.type(default)
 
     def get_default(self, run_id, run_defaults: dict = None):
         """Return default value for the option"""
@@ -179,13 +192,68 @@ class Option:
             value = config[self.name]
             if (self.type is not OMITTED
                     and not isinstance(value, self.type)):
-                raise InvalidConfiguration(
+                # TODO replace back with InvalidConfiguration
+                UserWarning(
                     f"Invalid type for option {self.name}. "
                     f"Excepted a {self.type}, got a {type(value)}")
         elif set_defaults:
             config[self.name] = self.get_default(run_id, run_defaults)
 
 
+# subclass Option for backward compatibility
+@export
+class Config(Option):
+    """An alternative to the `takes_config` class decorator
+       which uses the descriptor protocol to return the config
+       value when the attribute is accessed from within a plugin 
+    """
+    def __init__(self, **kwargs):
+        # for now set the name to empty string
+        # will be replaced by the actual name
+        # after __set_name__ is called on class
+        # instantiation
+        if 'name' not in kwargs:
+            kwargs['name'] = ''
+        super().__init__(**kwargs)
+
+    def __set_name__(self, owner, name):
+        ''''Plugin class has been instantiated
+            we can now set the option name and add it
+            to the plugins takes_config dictionary
+        '''
+        self.name = name
+        self.taken_by = owner.__name__
+        new_takes_config = {name: self}
+        if (hasattr(owner, 'takes_config')
+                and len(owner.takes_config)):
+            # Already have some options set, e.g. because of subclassing
+            # where both child and parent have a takes_config decorator
+            
+            if name in owner.takes_config:
+                raise RuntimeError(
+                    f"Attempt to specify option {name} twice")
+            owner.takes_config = immutabledict({
+                **owner.takes_config, **new_takes_config})
+        else:
+            owner.takes_config = immutabledict(new_takes_config)
+
+    def __get__(self, obj, objtype=None):
+        return self.fetch(obj)
+
+    def __set__(self, obj, value):
+        raise AttributeError(f"{self.name} is a plugin configuration and cannot be set directly.")
+
+    def fetch(self, plugin):
+        ''' This function is called when the attribute is being 
+        accessed. Should be overridden by subclasses to customize behavior.
+        '''
+        if not hasattr(plugin, 'config'):
+            raise AttributeError('Plugin has not been configured.')
+        if self.name in plugin.config:
+            return plugin.config[self.name]
+        return self.get_default(plugin.run_id)
+
+
 @export
 def combine_configs(old_config, new_config=None, mode='update'):
     if new_config is None:
