@@ -3,7 +3,6 @@
 A 'plugin' is something that outputs an array and gets arrays
 from one or more other plugins.
 """
-from concurrent.futures import wait
 from enum import IntEnum
 import inspect
 import itertools
@@ -13,7 +12,7 @@ import typing
 from warnings import warn
 from immutabledict import immutabledict
 import numpy as np
-
+from copy import copy, deepcopy
 import strax
 export, __all__ = strax.exporter()
 
@@ -44,7 +43,7 @@ class Plugin:
     You should NOT instantiate plugins directly.
     Do NOT add unpickleable things (e.g. loggers) as attributes.
     """
-    __version__ = '0.0.0'
+    __version__: typing.Optional[str] = '0.0.0'
 
     # For multi-output plugins these should be (immutable)dicts
     data_kind: typing.Union[str, immutabledict, dict]
@@ -65,7 +64,6 @@ class Plugin:
     # Meaningless if rechunk_on_save is False
     chunk_target_size_mb = strax.default_chunk_size_mb
 
-
     # For a source with online input (e.g. DAQ readers), crash if no new input
     # has appeared for this many seconds
     # This should be smaller than the mailbox timeout (which is intended as
@@ -118,12 +116,79 @@ class Plugin:
             del compute_pars[compute_pars.index('start')]
             del compute_pars[compute_pars.index('end')]
 
+        if not isinstance(self.save_when, (IntEnum, immutabledict, int)):
+            raise ValueError('save_when must be either a SaveWhen object or an immutabledict '
+                             'representing the different data_types provided.')
+
+        if hasattr(self, 'provides') and not isinstance(self.save_when, immutabledict):
+            # The ParallelSource plugin does not provide anything as it 
+            # inlines only already existing components, therefore we also do 
+            # not have to updated save_when
+            self.save_when = immutabledict.fromkeys(self.provides, self.save_when)
+
         self.compute_pars = compute_pars
         self.input_buffer = dict()
 
+    def __copy__(self, _deep_copy=False):
+        """
+        Copy main attributes that are set after __init__ by the context.
+
+        Note:
+            self.deps is NOT copied for it is recursive and therefor slow.
+            Instead, this is better handled within the context after all
+            plugins are copied.
+        """
+        plugin_copy = self.__class__()
+        plugin_copy.__init__()
+        # As explained in PR #485 only copy attributes whereof we know
+        # don't depend on the run_id (for use_per_run_defaults == False).
+        # Otherwise we might copy run-dependent things like to_pe.
+        for attribute in ['dtype',
+                          'lineage',
+                          'takes_config',
+                          '__version__',
+                          'config',
+                          'data_kind']:
+            source_value = getattr(self, attribute)
+            if _deep_copy:
+                plugin_copy.__setattr__(attribute, deepcopy(source_value))
+            else:
+                plugin_copy.__setattr__(attribute, copy(source_value))
+        return plugin_copy
+
+    def __deepcopy__(self):
+        return self.__copy__(_deep_copy=True)
+
+    def __getattr__(self, name):
+        """
+        allow access to config parameters as attributes
+        this allows backwards compatibility in cases where
+        a descriptor style config depends on a non descriptor
+        style config.
+        """
+        if name == 'config':
+            raise AttributeError('Plugin not configured yet.')
+        if hasattr(self, 'config') and name in self.config:
+            message = '''
+            Looks like you are mixing config paradigms,
+            this is not recommended.
+            '''
+            warn(message, UserWarning)
+            if isinstance(self.takes_config[name], strax.Config):
+                return self.takes_config[name].__get__(self)
+            return self.config[name]
+
+
+        raise AttributeError(f'{self.__class__.__name__} instance has no attribute {name}')
+
     def fix_dtype(self):
-        if not hasattr(self, 'dtype'):
+        try:
+            # Infer dtype should always precede self.dtype (e.g. due to
+            # copying)
             self.dtype = self.infer_dtype()
+        except RuntimeError:
+            if not hasattr(self, 'dtype'):
+                raise NotImplementedError(f'No dtype or infer_dtype specified')
 
         if self.multi_output:
             # Convert to a dict of numpy dtypes
@@ -169,12 +234,41 @@ class Plugin:
         # implementing all abstract methods...
         raise RuntimeError("No infer dtype method defined")
 
+    @property
+    def _auto_version(self):
+        """
+        Generate some auto-incremented version for the context hashing
+        system, see github.com/AxFoundation/strax/issues/217
+
+        Activate with setting __version__ to None
+        """
+        attributes = [attr for attr in self.__dir__()
+                      if not attr.startswith('__')]
+
+        def _return_hashable(attr):
+            if attr in ['takes_config', '_auto_version']:
+                # handled by context (or not worth tracking)
+                return
+            obj = getattr(self, attr)
+            try:
+                return strax.deterministic_hash(inspect.getsource(obj))
+            except TypeError:
+                pass
+            try:
+                return strax.deterministic_hash(obj)
+            except TypeError:
+                return str(obj)
+        res = {attr: _return_hashable(attr) for attr in attributes}
+        return 'auto_' + strax.deterministic_hash(res)
+
     def version(self, run_id=None):
         """Return version number applicable to the run_id.
         Most plugins just have a single version (in .__version__)
         but some may be at different versions for different runs
         (e.g. time-dependent corrections).
         """
+        if self.__version__ is None:
+            return self._auto_version
         return self.__version__
 
     def __repr__(self):
@@ -354,7 +448,7 @@ class Plugin:
                     # If any of the inputs were trimmed due to early splits,
                     # trim the others too.
                     # In very hairy cases this can take multiple passes.
-                    # TODO: can we optimize this, or code it more elegantly?
+                    # can we optimize this, or code it more elegantly?
                     max_passes_left = 10
                     while max_passes_left > 0:
                         this_chunk_end = min([x.end for x in inputs.values()]
@@ -404,7 +498,11 @@ class Plugin:
                         f"Plugin {d} terminated without fetching last {d}!")
 
             # This can happen especially in time range selections
-            if int(self.save_when) != strax.SaveWhen.NEVER:
+            if hasattr(self.save_when, 'values'):
+                save_when = max([int(save_when) for save_when in self.save_when.values()])
+            else:
+                save_when = self.save_when
+            if save_when > strax.SaveWhen.EXPLICIT:
                 for d, buffer in self.input_buffer.items():
                     # Check the input buffer is empty
                     if buffer is not None and len(buffer):
@@ -462,7 +560,13 @@ class Plugin:
 
             # For non-saving plugins, don't be strict, just take whatever
             # endtimes are available and don't check time-consistency
-            if int(self.save_when) == strax.SaveWhen.NEVER:
+            # Side mark this wont work for a plugin which has a SaveWhen.NEVER and other
+            # SaveWhen type.
+            if hasattr(self.save_when, 'values'):
+                save_when = max([int(save_when) for save_when in self.save_when.values()])
+            else:
+                save_when = self.save_when
+            if save_when <= strax.SaveWhen.EXPLICIT:
                 # </start>This warning/check will be deleted, see UserWarning
                 if len(set(tranges.values())) != 1:
                     end = max([v.end for v in kwargs.values()])  # Don't delete
@@ -591,10 +695,10 @@ class OverlapWindowPlugin(Plugin):
             raise RuntimeError("OverlapWindowPlugin must have a dependency")
 
         # Add cached inputs to compute arguments
-        for k, v in kwargs.items():
+        for data_kind, chunk in kwargs.items():
             if len(self.cached_input):
-                kwargs[k] = strax.Chunk.concatenate(
-                    [self.cached_input[k], v])
+                kwargs[data_kind] = strax.Chunk.concatenate(
+                    [self.cached_input[data_kind], chunk])
 
         # Compute new results
         result = super().do_compute(chunk_i=chunk_i, **kwargs)
@@ -604,7 +708,7 @@ class OverlapWindowPlugin(Plugin):
                                  allow_early_split=False)
 
         # When does this batch of inputs end?
-        ends = [v.end for v in kwargs.values()]
+        ends = [c.end for c in kwargs.values()]
         if not len(set(ends)) == 1:
             raise RuntimeError(
                 f"OverlapWindowPlugin got incongruent inputs: {kwargs}")
@@ -627,10 +731,32 @@ class OverlapWindowPlugin(Plugin):
         # Again, take a bit of overkill for good measure
         cache_inputs_beyond = int(self.sent_until
                                   - 2 * self.get_window_size() - 1)
-        for k, v in kwargs.items():
-            _, self.cached_input[k] = v.split(t=cache_inputs_beyond,
-                                              allow_early_split=True)
 
+        # Cache inputs, make sure that the chunks start at the same time to
+        # prevent issues in input buffers later on
+        prev_split = cache_inputs_beyond
+        max_trials = 10
+        for try_counter in range(max_trials):
+            for data_kind, chunk in kwargs.items():
+                _, self.cached_input[data_kind] = chunk.split(
+                    t=prev_split,
+                    allow_early_split=True)
+                prev_split = self.cached_input[data_kind].start
+
+            unique_starts = set([c.start for c in self.cached_input.values()])
+            chunk_starts_are_equal = len(unique_starts) == 1
+            if chunk_starts_are_equal:
+                self.log.debug(
+                    f'Success after {try_counter}. '
+                    f'Extra time = {cache_inputs_beyond-prev_split} ns')
+                break
+            else:
+                self.log.debug(
+                    f'Inconsistent start times of the cashed chunks after'
+                    f' {try_counter}/{max_trials} passes.\nChunks {self.cached_input}')
+        else:
+            raise ValueError(f'Buffer start time inconsistency cannot be '
+                             f'resolved after {max_trials} tries')
         return result
 
 
@@ -638,6 +764,16 @@ class OverlapWindowPlugin(Plugin):
 class LoopPlugin(Plugin):
     """Plugin that disguises multi-kind data-iteration by an event loop
     """
+    # time_selection: Kind of time selection to apply:
+    # - touching: select things that (partially) overlap with the range.
+    # NB! Use this option with care since if e.g. two events are
+    # adjacent, touching windows might return ambiguous results as peaks
+    # may be touching both events.
+    # The number of samples to be desired to overlapped can be set by
+    # self.touching_window. Otherwise 0 is assumed (see strax.touching_windows)
+    # - fully_contained: (default) select things fully contained in the range
+    time_selection = 'fully_contained'
+
     def compute(self, **kwargs):
         # If not otherwise specified, data kind to loop over
         # is that of the first dependency (e.g. events)
@@ -670,7 +806,23 @@ class LoopPlugin(Plugin):
                          for x in examples]))
 
             if k != loop_over:
-                r = strax.split_by_containment(things, base)
+                if self.time_selection == 'fully_contained':
+                    r = strax.split_by_containment(things, base)
+                elif self.time_selection == 'touching':
+                    # Experimental feature that should be handled with care:
+                    # github.com/AxFoundation/strax/pull/424
+                    warn(f'{self.__class__.__name__} has a touching time '
+                         f'selection. This may lead to ambiguous results as two '
+                         f'{loop_over}\'s may contain the same {k}, thereby a '
+                         f'given {k} can be included multiple times.')
+                    window = 0
+                    if hasattr(self, 'touching_window'):
+                        window = self.touching_window
+                    r = strax.split_touching_windows(things,
+                                                     base,
+                                                     window=window)
+                else:
+                    raise RuntimeError('Unknown time_selection')
                 if len(r) != len(base):
                     raise RuntimeError(f"Split {k} into {len(r)}, "
                                        f"should be {len(base)}!")
@@ -732,11 +884,25 @@ class LoopPlugin(Plugin):
 @export
 class CutPlugin(Plugin):
     """Generate a plugin that provides a boolean for a given cut specified by 'cut_by'"""
-    save_when = SaveWhen.NEVER
+    save_when = SaveWhen.TARGET
 
     def __init__(self):
         super().__init__()
-
+        
+        compute_pars = list(
+            inspect.signature(self.cut_by).parameters.keys())
+        if 'chunk_i' in compute_pars:
+            self.compute_takes_chunk_i = True
+            del compute_pars[compute_pars.index('chunk_i')]
+        if 'start' in compute_pars:
+            if 'end' not in compute_pars:
+                raise ValueError(f"Compute of {self} takes start, "
+                                 f"so it should also take end.")
+            self.compute_takes_start_end = True
+            del compute_pars[compute_pars.index('start')]
+            del compute_pars[compute_pars.index('end')]
+        self.compute_pars = compute_pars
+        
         _name = strax.camel_to_snake(self.__class__.__name__)
         if not hasattr(self, 'provides'):
             self.provides = _name
@@ -753,7 +919,7 @@ class CutPlugin(Plugin):
     def infer_dtype(self):
         dtype = [(self.cut_name, np.bool_, self.cut_description)]
         # Alternatively one could use time_dt_fields for low level plugins.
-        dtype = dtype + strax.time_fields
+        dtype = strax.time_fields + dtype
         return dtype
 
     def compute(self, **kwargs):
@@ -788,7 +954,7 @@ class CutPlugin(Plugin):
 class MergeOnlyPlugin(Plugin):
     """Plugin that merges data from its dependencies
     """
-    save_when = SaveWhen.NEVER
+    save_when = SaveWhen.EXPLICIT
 
     def infer_dtype(self):
         deps_by_kind = self.dependencies_by_kind()
@@ -815,11 +981,15 @@ class ParallelSourcePlugin(Plugin):
     while multiprocessing.
     """
     parallel = 'process'
+    # should we set this here?
+    input_timeout = 300
 
     @classmethod
     def inline_plugins(cls, components, start_from, log):
         plugins = components.plugins.copy()
-
+        loader_plugins = components.loader_plugins.copy()
+        log.debug(f'Try to inline plugins starting from {start_from}')
+        
         sub_plugins = {start_from: plugins[start_from]}
         del plugins[start_from]
 
@@ -840,7 +1010,7 @@ class ParallelSourcePlugin(Plugin):
             else:
                 # No more plugins we can inline
                 break
-
+        log.debug(f'Trying to inline the following sub-plugins: {sub_plugins}')
         if len(set(list(sub_plugins.values()))) == 1:
             # Just one plugin to inline: no use
             log.debug("Just one plugin to inline: skipping")
@@ -911,13 +1081,16 @@ class ParallelSourcePlugin(Plugin):
             p.dtype = p.sub_plugins[to_send].dtype_for(to_send)
         for d in p.provides:
             plugins[d] = p
-        p.deps = {d: plugins[d] for d in p.depends_on}
+        
+        log.debug(f'Trying to find plugins for dependencies: {p.depends_on}')
+        
+        p.deps = {d: plugins[d] if plugins.get(d, None) else loader_plugins[d] for d in p.depends_on}
 
         log.debug(f"Inlined plugins: {p.sub_plugins}."
                   f"Inlined savers: {p.sub_savers}")
 
         return strax.ProcessorComponents(
-            plugins, components.loaders, savers, components.targets)
+            plugins, components.loaders, components.loader_plugins, savers, components.targets)
 
     def __init__(self, depends_on):
         self.depends_on = depends_on
