@@ -2,15 +2,48 @@ import numpy as np
 import numba
 
 import strax
+from strax.processing.general import _touching_windows
+
 export, __all__ = strax.exporter()
 
 # Hardcoded numbers:
-TRIAL_COUNTER_NEIGHBORING_RECORDS = 100  # Trial counter when looking for hitlet data.
 NO_FWXM = -42  # Value in case FWXM cannot be found.
 
-# ----------------------
-# Hitlet building:
-# ----------------------
+
+@export
+def create_hitlets_from_hits(hits,
+                             save_outside_hits,
+                             channel_range,
+                             chunk_start=0,
+                             chunk_end=np.inf,):
+    """
+    Function which creates hitlets from a bunch of hits.
+
+    :param hits: Hits found in records.
+    :param save_outside_hits: Tuple with left and right hit extension.
+    :param channel_range: Detectors change range from channel map.
+    :param chunk_start: (optional) start time of a chunk. Ensures that
+        no hitlet is earlier than this timestamp.
+    :param chunk_end: (optional) end time of a chunk. Ensures that
+        no hitlet ends later than this timestamp.
+
+    :return: Hitlets with temporary fields (data, max_goodness_of_split...)
+    """
+    # Merge concatenate overlapping  within a channel. This is important
+    # in case hits were split by record boundaries. In case we
+    # accidentally concatenate two PMT signals we split them later again.
+    hits = strax.concat_overlapping_hits(hits,
+                                         save_outside_hits,
+                                         channel_range,
+                                         chunk_start,
+                                         chunk_end, )
+    hits = strax.sort_by_time(hits)
+
+    hitlets = np.zeros(len(hits), strax.hitlet_dtype())
+    strax.copy_to_buffer(hits, hitlets, '_refresh_hit_to_hitlets')
+    return hitlets
+
+
 @export
 def concat_overlapping_hits(hits, extensions, pmt_channels, start, end):
     """
@@ -18,8 +51,7 @@ def concat_overlapping_hits(hits, extensions, pmt_channels, start, end):
     right hit extension. Assumes that hits are sorted correctly.
 
     Note:
-        This function only updates time, length and record_i of the hit.
-        (record_i is set according to the first hit)
+        This function only updates time, and length of the hit.
 
     :param hits: Hits in records.
     :param extensions: Tuple of the left and right hit extension.
@@ -30,7 +62,6 @@ def concat_overlapping_hits(hits, extensions, pmt_channels, start, end):
     :returns:
         array with concataneted hits.
     """
-    # Getting channel map and compute the number of channels:
     first_channel, last_channel = pmt_channels
     nchannels = last_channel - first_channel + 1
 
@@ -42,7 +73,8 @@ def concat_overlapping_hits(hits, extensions, pmt_channels, start, end):
                                                'endtime'), np.int64)]))
 
     if len(hits):
-        hits = _concat_overlapping_hits(hits, extensions, first_channel, last_hit_in_channel, start, end)
+        hits = _concat_overlapping_hits(
+            hits, extensions, first_channel, last_hit_in_channel, start, end)
     return hits
 
 
@@ -51,307 +83,192 @@ def concat_overlapping_hits(hits, extensions, pmt_channels, start, end):
 def _concat_overlapping_hits(hits,
                              extensions,
                              first_channel,
-                             last_hit_in_channel,
-                             start=0,
-                             end=float('inf'),
+                             last_hit_in_channel_buffer,
+                             chunk_start=0,
+                             chunk_end=float('inf'),
                              _result_buffer=None):
     buffer = _result_buffer
-    offset = 0
+    res_offset = 0
 
-    le, re = extensions
+    left_extension, right_extension = extensions
     dt = hits['dt'][0]
     assert np.all(hits['dt'] == dt), 'All hits must have the same dt!'
 
-    for h in hits:
-        st = h['time'] - int(le * h['dt'])
-        et = strax.endtime(h) + int(re * h['dt'])
-        hc = h['channel']
-        r_i = h['record_i']
+    for hit in hits:
+        time_with_le = hit['time'] - int(left_extension * hit['dt'])
+        endtime_with_re = strax.endtime(hit) + int(right_extension * hit['dt'])
+        hit_channel = hit['channel']
 
-        lhc = last_hit_in_channel[hc - first_channel]
-        # Have not found any hit in this channel yet:
-        if lhc['time'] == 0:
-            lhc['time'] = max(st, start)
-            lhc['endtime'] = min(et, end)
-            lhc['channel'] = hc
-            lhc['record_i'] = h['record_i']
-            lhc['dt'] = dt
+        last_hit_in_channel = last_hit_in_channel_buffer[hit_channel - first_channel]
 
-        # Checking if events overlap:
+        found_no_hit_for_channel_yet = last_hit_in_channel['time'] == 0
+        if found_no_hit_for_channel_yet:
+            last_hit_in_channel['time'] = max(time_with_le, chunk_start)
+            last_hit_in_channel['endtime'] = min(endtime_with_re, chunk_end)
+            last_hit_in_channel['channel'] = hit_channel
+            last_hit_in_channel['dt'] = dt
         else:
-            if lhc['endtime'] >= st:
-                # Yes, so we have to update only the end_time:
-                lhc['endtime'] = et
+            hits_overlap_in_channel = last_hit_in_channel['endtime'] >= time_with_le
+            if hits_overlap_in_channel:
+                last_hit_in_channel['endtime'] = endtime_with_re
             else:
                 # No, this means we have to save the previous data and update lhc:
-                res = buffer[offset]
-                res['time'] = lhc['time']
-                res['length'] = (lhc['endtime'] - lhc['time']) // lhc['dt']
-                res['channel'] = lhc['channel']
-                res['record_i'] = lhc['record_i']
-                res['dt'] = lhc['dt']
-                offset += 1
-                if offset == len(buffer):
-                    yield offset
-                    offset = 0
-
+                res = buffer[res_offset]
+                res['time'] = last_hit_in_channel['time']
+                hitlet_length = (last_hit_in_channel['endtime'] - last_hit_in_channel['time'])
+                hitlet_length //= last_hit_in_channel['dt']
+                res['length'] = hitlet_length
+                res['channel'] = last_hit_in_channel['channel']
+                res['dt'] = last_hit_in_channel['dt']
+                
                 # Updating current last hit:
-                lhc['time'] = st
-                lhc['endtime'] = et
-                lhc['channel'] = hc
-                lhc['record_i'] = r_i
+                last_hit_in_channel['time'] = time_with_le
+                last_hit_in_channel['endtime'] = endtime_with_re
+                last_hit_in_channel['channel'] = hit_channel
+                
+                res_offset += 1
+                if res_offset == len(buffer):
+                    yield res_offset
+                    res_offset = 0
 
     # We went through so now we have to save all remaining hits:
-    mask = last_hit_in_channel['time'] != 0
-    for lhc in last_hit_in_channel[mask]:
-        res = buffer[offset]
-        res['time'] = lhc['time']
-        res['channel'] = lhc['channel']
-        res['length'] = (lhc['endtime'] - lhc['time']) // lhc['dt']
-        res['record_i'] = lhc['record_i']
-        res['dt'] = lhc['dt']
-        offset += 1
-        if offset == len(buffer):
-            yield offset
-            offset = 0
-    yield offset
+    mask = last_hit_in_channel_buffer['time'] != 0
+    for last_hit_in_channel in last_hit_in_channel_buffer[mask]:
+        res = buffer[res_offset]
+        res['time'] = last_hit_in_channel['time']
+        res['channel'] = last_hit_in_channel['channel']
+        hitlet_length = (last_hit_in_channel['endtime'] - last_hit_in_channel['time'])
+        hitlet_length //= last_hit_in_channel['dt']
+        res['length'] = hitlet_length
+        res['dt'] = last_hit_in_channel['dt']
+        res_offset += 1
+        if res_offset == len(buffer):
+            yield res_offset
+            res_offset = 0
+    yield res_offset
 
 
 @export
-@numba.njit(nogil=True, cache=True)
-def refresh_hit_to_hitlets(hits, hitlets):
-    """
-    Function which copies basic hit information into a new hitlet array.
-    """
-    nhits = len(hits)
-    for ind in range(nhits):
-        h_new = hitlets[ind]
-        h_old = hits[ind]
-
-        h_new['time'] = h_old['time']
-        h_new['length'] = h_old['length']
-        h_new['channel'] = h_old['channel']
-        h_new['area'] = h_old['area']
-        h_new['record_i'] = h_old['record_i']
-        h_new['dt'] = h_old['dt']
-
-
-@export
-@numba.njit(nogil=True, cache=True)
-def get_hitlets_data(hitlets, records, to_pe):
+def get_hitlets_data(hitlets, records, to_pe, min_hitlet_sample=200):
     """
     Function which searches for every hitlet in a given chunk the 
-    corresponding records data.
-    
+    corresponding records data. Additionally compute the total area of
+    the signal.
+
     :param hitlets: Hitlets found in a chunk of records.
     :param records: Records of the chunk.
-    :param to_pe: Array with area conversion factors from adc/sample to 
-        pe/sample
-    
-    Note:
-        hitlets must have a "data" and "area" field.
-    
-    The function updates the hitlet fields time, length (if necessary 
-    e.g. hit was extended in regions of now records) and area
-    according to the found data.
+    :param to_pe: Array with area conversion factors from adc/sample to
+        pe/sample. Please make sure that to_pe has the correct shape.
+        The array index should match the channel number.
+    :param min_hitlet_sample: minimal length of the hitlet data field.
+        prevents numba compiling from running into race conditions.
+    :returns: Hitlets including data stored in the "data" field
+        (if it did not exists before it will be added.)
     """
+    if len(hitlets) == 0:
+        return np.zeros(0, dtype=strax.hitlet_with_data_dtype(min_hitlet_sample))
 
-    rlink = strax.record_links(records)
-    for h in hitlets:
-        data, start_time = get_single_hitlet_data(h, records, *rlink)
-        h['length'] = len(data)
-        h['data'][:len(data)] = data * to_pe[h['channel']]
-        h['time'] = start_time
-        h['area'] = np.sum(data * to_pe[h['channel']])
+    if len(hitlets) > 0 and len(records) == 0:
+        raise ValueError('Cannot get data for hitlets if records are empty!')
+
+    # Numba will not raise any exceptions if to_pe is too short, leading
+    # to strange bugs.
+    to_pe_has_wrong_shape = len(to_pe) < hitlets['channel'].max()
+    if to_pe_has_wrong_shape:
+        raise ValueError('"to_pe" has a wrong shape. Array index must'
+                         ' match channel numbers.')
+
+    hitelts_is_single_row = isinstance(hitlets, np.void)
+    if hitelts_is_single_row:
+        # A structured array becomes void type if a single row is called,
+        # e.g. hitlets[0] which does not work in numba while, hitlets[:1]
+        # does. So we have to convert the row into the correct format first.
+        hitlets = np.array([hitlets])
+
+    data_field_in_hitlets = 'data' in hitlets.dtype.names
+    if data_field_in_hitlets:
+        data_is_not_empty = np.any(hitlets['data'] != 0)
+        if data_is_not_empty:
+            raise ValueError('The data field of hitlets must be empty!')
+
+        data_field_not_long_enough = len(hitlets[0]['data']) < hitlets['length'].max()
+        if data_field_not_long_enough:
+            raise ValueError('The data field must be as large as the longest hitlet in our data.')
+
+        hitlets_with_data_field = hitlets
+    else:
+        n_samples = max(min_hitlet_sample, hitlets['length'].max())
+        hitlets_with_data_field = np.zeros(len(hitlets), strax.hitlet_with_data_dtype(n_samples))
+        strax.copy_to_buffer(hitlets,
+                             hitlets_with_data_field,
+                             '_copy_hitlets_to_hitlets_width_data')
+
+    _get_hitlets_data(hitlets_with_data_field, records, to_pe)
+    if np.any(hitlets_with_data_field['length'] == 0):
+        raise ValueError('Try to create zero length hitlets which is forbidden!')
+        
+    return hitlets_with_data_field
 
 
-@export
-@numba.njit(nogil=True, cache=True)
-def get_single_hitlet_data(hitlet, records, prev_r, next_r):
-    """
-    Function which gets the data of a single hit or hitlet. The data is
-    returned according to the objects time and length (LE/RE is not
-    included in case of a hit.).
+@numba.jit(nopython=True, nogil=True, cache=True)
+def _get_hitlets_data(hitlets, records, to_pe):
+    rranges = _touching_windows(records['time'],
+                                strax.endtime(records),
+                                hitlets['time'],
+                                strax.endtime(hitlets))
 
-    In case the hit or hitlet is extended into non-recorded regions
-    the data gets chopped.
+    for i, h in enumerate(hitlets):
+        recorded_samples_offset = 0
+        n_recorded_samples = 0
+        is_first_record = True
+        for ind, r_ind in enumerate(range(rranges[i][0], rranges[i][1])):
+            r = records[r_ind]
+            if r['channel'] != h['channel']:
+                continue
 
+            (r_start, r_end), (h_start, h_end) = strax.overlap_indices(
+                r['time'] // r['dt'],
+                r['length'],
+                h['time'] // h['dt'],
+                h['length'])
+            
+            if (r_end - r_start) == 0 and (h_end - h_start) == 0:
+                # _touching_windows will give a range of overlapping records  with hitlet
+                # independent of channel. Hence, in rare cases it might be that a record of
+                # channel A touches with a hitlet of channel B which  starts before the previous
+                # record of channel b. Hence we get one non-overlapping record in channel b.
+                continue   
+            
+            if is_first_record:
+                # We need recorded_samples_offset because hits may extend beyond the boundaries
+                # of our recorded data. As the data is not defined in those regions we have to
+                # chop and realign our data. See the following Example: (fragment 0, 1) [2, 2, 2,
+                # 2] [2, 2, 2] with a hitfinder threshold of 1 and left/right extension of 3. In
+                # the first fragment our hitlet would range from 3 to 8 in the second from 8 to
+                # 11. Hence we have to subtract from every h_start and h_end the offset of 3 to
+                # realign our data. Time and length of the hitlet are updated accordingly.
+                is_first_record = False
+                recorded_samples_offset = h_start
 
-    :param hitlet: Hits or hitlets.
-    :param records: Records
-    :param prev_r: Index of the previous record seen from the current
-        record. (Return of strax.record_links)
-    :param next_r: Index of the next record seen by the current record.
-        (Return of strax.record_links)
-    :return:
-        np.ndarray: Samples of the hitlet [ADC]
-        int: Start time of the hitlet. (In case data gets chopped on the
-            left)
-    """
-    temp_data = np.zeros(hitlet['length'], dtype=np.float64)
+            h_start -= recorded_samples_offset
+            h_end -= recorded_samples_offset
 
-    # Lets get the starting record and the corresponding data:
-    r_i = hitlet['record_i']
-    r = records[r_i]
-    data, (p_start_i, p_end_i) = _get_thing_data(hitlet, r)
-    temp_data[p_start_i:p_end_i] = data
+            h['data'][h_start: h_end] += r['data'][r_start: r_end] + r['baseline'] % 1
+            n_recorded_samples += r_end - r_start
 
-    # We have to store the first and last index of the so far found data 
-    data_start = p_start_i
-    data_end = p_end_i
+        # Chop time and length in case hit extends into non-recorded regions.
+        h['time'] += int(recorded_samples_offset * h['dt'])
+        h['length'] = n_recorded_samples
 
-    if not (p_end_i - p_start_i == hitlet['length']):
-        # We have not found the entire data yet....
-        # Starting with data before our current record:
-        trial_counter = 0
-        prev_r_i = r_i
-        while p_start_i:
-            # We are still searching for data in a previous record.
-            temp_prev_r_i = prev_r[prev_r_i]
-            if temp_prev_r_i == -1:
-                # There is no (more) previous record. So stop here and keep
-                # last pre_r_i
-                break
-            prev_r_i = temp_prev_r_i
-
-            # There is a previous record:
-            r = records[prev_r_i]
-            data, (p_start_i, end) = _get_thing_data(hitlet, r)
-            if not end:
-                # If end is zero this means we have not found any
-                # overlap which should not have happened. In case of an
-                # overlap start and end should reflect the start and end
-                # sample of the hitlet for which we found data.
-                print('Data found for this record:', data,
-                      'Start index', p_start_i,
-                      'End index:', end)
-                raise ValueError('This is odd found previous record, but no'
-                                 ' overlapping indices.')
-            temp_data[p_start_i:data_start] = data
-            data_start = p_start_i
-
-            if trial_counter > TRIAL_COUNTER_NEIGHBORING_RECORDS:
-                raise RuntimeError('Tried too hard. There are more than'
-                                   '100 successive records. This is odd...')
-            trial_counter += 1
-
-        # Now we have to do the very same for records in the future:
-        # Almost the same code as above can I change this?
-        trial_counter = 0
-        next_r_i = r_i
-        while hitlet['length'] - p_end_i:
-            # We are still searching for data in a next record.
-            temp_next_r_i = next_r[next_r_i]
-            if temp_next_r_i == -1:
-                # There is no (more) previous record. So stop here and keep
-                # last next_r_i
-                break
-            next_r_i = temp_next_r_i
-            # There is a next record:
-            r = records[next_r_i]
-            data, (start, p_end_i) = _get_thing_data(hitlet, r)
-            if not start:
-                # If start is zero this means we have not found any
-                # overlap which should not have happened. In case of an
-                # overlap start and end should reflect the start and end
-                # sample of the hitlet for which we found data.
-                print('Data found for this record:', data,
-                      'Start index', start,
-                      'End index:', p_end_i)
-                raise ValueError('This is odd found the next record, but no'
-                                 ' overlapping indicies.')
-            temp_data[data_end:p_end_i] = data
-            data_end = p_end_i
-
-            if trial_counter > TRIAL_COUNTER_NEIGHBORING_RECORDS:
-                raise RuntimeError('Tried too hard. There are more than'
-                                   '100 successive records. This is odd...')
-            trial_counter += 1
-
-    # In some cases it might have happened that due to the left and right hit extension
-    # we extended our hitlet into regions without any data so we have to chop
-    # "time" according to the data we found....
-    time = hitlet['time'] + data_start * hitlet['dt']
-    temp_data = temp_data[data_start:data_end] + r['baseline'] % 1
-    return temp_data, time
-
-
-@numba.njit(nogil=True, cache=True)
-def _get_thing_data(thing, container):
-    """
-    Function which returns data for some overlapping indices of a thing
-    in a container. 
-    
-    Note:
-        Thing must be of the interval dtype kind.
-    """
-    overlap_hit_i, overlap_record_i = strax.overlap_indices(thing['time']//thing['dt'],
-                                                            thing['length'],
-                                                            container['time']//container['dt'],
-                                                            container['length'])
-    data = container['data'][overlap_record_i[0]:overlap_record_i[1]]
-    return data, overlap_hit_i
-
-# ----------------------
-# Hitlet splitting:
-# ----------------------
-@export
-def update_new_hitlets(hitlets, records, next_ri, to_pe):
-    """
-    Function which computes the hitlet data area and record_i after
-    splitting.
-
-    :param hitlets: New hitlets received after splitting.
-    :param records: Records of the chunk.
-    :param next_ri: Index of next record for current record record_i.
-    :param  to_pe: ADC to PE conversion factor array (of n_channels).
-    """
-    _update_record_i(hitlets, records, next_ri)
-    get_hitlets_data(hitlets, records, to_pe)
-
-
-@numba.njit(cache=True, nogil=True)
-def _update_record_i(new_hitlets, records, next_ri):
-    """
-    Function which updates the record_i value of the new hitlets. 
-    
-    Notes:
-        Assumes new_hitlets to be sorted in time.
-    """
-    for ind, hit in enumerate(new_hitlets):
-
-        updated = False
-        counter = 0
-        current_ri = hit['record_i']
-        while not updated:
-            r = records[current_ri]
-            # Hitlet must only partially be contained in record_i:
-            time = hit['time']
-            end_time = strax.endtime(hit)
-            start_in = (r['time'] <= time) & (time < strax.endtime(r))
-            end_in = (r['time'] < end_time) & (end_time <= strax.endtime(r))
-            if start_in or end_in:
-                hit['record_i'] = current_ri
-                break
-            else:
-                last_ri = current_ri
-                current_ri = next_ri[current_ri]
-                counter += 1
-                
-            if current_ri == -1:
-                print('Record:\n', r, '\nHit:\n', hit)
-                raise ValueError('Was not able to find record_i')
-
-            if counter > TRIAL_COUNTER_NEIGHBORING_RECORDS:
-                print(ind, last_ri)
-                raise RuntimeError('Tried too often to find correct record_i.')
+        h['data'][:] = h['data'][:] * to_pe[h['channel']]
+        h['area'] = np.sum(h['data'])
 
 
 # ----------------------
 # Hitlet properties:
 # ----------------------
 @export
-@numba.njit(cache=True, nogil=True)
+@numba.njit(cache=False, nogil=True)
 def hitlet_properties(hitlets):
     """
     Computes additional hitlet properties such as amplitude, FHWM, etc.
@@ -359,7 +276,7 @@ def hitlet_properties(hitlets):
     for ind, h in enumerate(hitlets):
         dt = h['dt']
         data = h['data'][:h['length']]
-        
+
         if not np.any(data):
             continue
 
@@ -398,7 +315,7 @@ def hitlet_properties(hitlets):
             h['range_80p_area'] = res[3]-res[0]
             
         # Compute width based on HDR:
-        resh = highest_density_region_width(h['data'], 
+        resh = highest_density_region_width(data, 
                                             fractions_desired=np.array([0.5, 0.8]),
                                             dt=h['dt'],
                                             fractionl_edges=True,
@@ -410,7 +327,6 @@ def hitlet_properties(hitlets):
         h['range_hdr_80p_area'] = resh[1,1]-resh[1,0]
 
 
-
 @export
 @numba.njit(cache=True, nogil=True)
 def get_fwxm(hitlet, fraction=0.5):
@@ -434,7 +350,7 @@ def get_fwxm(hitlet, fraction=0.5):
 
     index_maximum = np.argmax(data)
     max_val = data[index_maximum] * fraction
-    if np.all(data > max_val) or np.all(data == 0):
+    if np.all(data > max_val) or np.all(data <= 0):
         # In case all samples are larger, FWXM is not definition.
         return np.nan, np.nan
 
@@ -448,6 +364,8 @@ def get_fwxm(hitlet, fraction=0.5):
         # before maximum and go left
         lbi = (index_maximum - 1) - lbi  # start sample minus samples we went to the left
         m = data[lbi + 1] - lbs  # divided by 1 sample
+        if m == 0:
+            return np.nan, np.nan
         left_edge = lbi + (max_val - lbs) / m + 0.5
     else:
         # There is no data before the maximum:
@@ -458,6 +376,8 @@ def get_fwxm(hitlet, fraction=0.5):
         rbi, rbs = _get_fwxm_boundary(post_max, max_val)  # Starting after maximum and go right
         rbi += 1 + index_maximum  # sample to the right plus start
         m = data[rbi - 1] - rbs
+        if m == 0:
+            return np.nan, np.nan
         right_edge = rbi - (max_val - rbs) / m + 0.5
     else:
         right_edge = len(data)
@@ -488,6 +408,7 @@ def _get_fwxm_boundary(data, max_val):
             return ind, s
     return len(data)-1, data[-1]
 
+
 @export
 def conditional_entropy(hitlets, template='flat', square_data=False):
     """
@@ -594,12 +515,12 @@ def _conditional_entropy(hitlets, template, flat=False, square_data=False):
     return res
 
 
-@numba.njit
+@numba.njit(cache=True)
 def highest_density_region_width(data,
-                                  fractions_desired,
-                                  dt=1,
-                                  fractionl_edges=False,
-                                  _buffer_size=100):
+                                 fractions_desired,
+                                 dt=1,
+                                 fractionl_edges=False,
+                                 _buffer_size=100):
     """
     Function which computes the left and right edge based on the outer
     most sample for the highest density region of a signal.
@@ -615,19 +536,34 @@ def highest_density_region_width(data,
     :param fractionl_edges: If true computes width as fractional time
         depending on the covered area between the current and next
         sample.
-    :param _buffer_size: Maximal number of allowed intervals.
+    :param _buffer_size: Maximal number of allowed intervals. If signal
+        exceeds number e.g. due to noise width computation is skipped.
     """
     res = np.zeros((len(fractions_desired), 2), dtype=np.float32)
     data = np.maximum(data, 0)
-    inter, amps = strax.highest_density_region(data, fractions_desired, _buffer_size=_buffer_size)
+    
+    if np.all(data == 0):
+        res[:] = np.nan
+        return res
+
+    inter, amps = strax.highest_density_region(data,
+                                               fractions_desired,
+                                               only_upper_part=True,
+                                               _buffer_size=_buffer_size,
+                                               )
+
+    for index_area_fraction, (interval_indicies, area_fraction_amplitude) in enumerate(zip(inter, amps)):
+        if np.all(interval_indicies[:] == -1):
+            res[index_area_fraction, :] = np.nan
+            continue
 
-    for f_ind, (i, a) in enumerate(zip(inter, amps)):
         if not fractionl_edges:
-            res[f_ind, 0] = i[0, 0] * dt
-            res[f_ind, 1] = i[1, np.argmax(i[1, :])] * dt
+            res[index_area_fraction, 0] = interval_indicies[0, 0] * dt
+            res[index_area_fraction, 1] = interval_indicies[1, np.argmax(interval_indicies[1, :])] * dt
         else:
-            left = i[0, 0]
-            right = i[1, np.argmax(i[1, :])] - 1  # since value corresponds to outer edge
+            left = interval_indicies[0, 0]
+            # -1 since value corresponds to outer edge:
+            right = interval_indicies[1, np.argmax(interval_indicies[1, :])] - 1
 
             # Get amplitudes of outer most samples
             # and amplitudes of adjacent samples (if any)
@@ -641,11 +577,13 @@ def highest_density_region_width(data,
             if (right + 1) < len(data):
                 next_right_amp = data[right + 1]
 
-            # Compute fractions and new left and right edges:
-            fl = (left_amp - a) / (left_amp - next_left_amp)
-            fr = (right_amp - a) / (right_amp - next_right_amp)
+            # Compute fractions and new left and right edges, the case
+            # left_amp == next_left_amp cannot occure by the definition of the highest density
+            # region.
+            fl = (left_amp - area_fraction_amplitude) / (left_amp - next_left_amp)
+            fr = (right_amp - area_fraction_amplitude) / (right_amp - next_right_amp)
 
-            res[f_ind, 0] = (left + 0.5 - fl) * dt
-            res[f_ind, 1] = (right + 0.5 + fr) * dt
+            res[index_area_fraction, 0] = (left + 0.5 - fl) * dt
+            res[index_area_fraction, 1] = (right + 0.5 + fr) * dt
 
     return res
