@@ -71,8 +71,8 @@ def merge_peaks(peaks, start_merge_at, end_merge_at,
         new_p['n_saturated_channels'] = new_p['saturated_channel'].sum()
 
         # Use the tight coincidence of the peak with the highest amplitude
-        new_p['tight_coincidence'] = old_peaks['tight_coincidence'][
-            old_peaks['data'].max(axis=1).argmax()]
+        i_max_subpeak = old_peaks['data'].max(axis=1).argmax()
+        new_p['tight_coincidence'] = old_peaks['tight_coincidence'][i_max_subpeak]
         
         # If the endtime was in the peaks we have to recompute it here 
         # because otherwise it will stay set to zero due to the buffer
@@ -136,3 +136,30 @@ def _replace_merged(result, orig, merge, skip_windows):
 
     assert result_i == len(result)
     assert window_i == len(skip_windows)
+
+    
+@export
+@numba.njit(cache=True, nogil=True)
+def add_lone_hits(peaks, lone_hits, to_pe):
+    """
+    Function which adds information from lone hits to peaks if lone hit
+    is inside a peak (e.g. after merging.). Modifies peak area and data
+    inplace.
+
+    :param peaks: Numpy array of peaks
+    :param lone_hits: Numpy array of lone_hits
+    :param to_pe: Gain values to convert lone hit area into PE.
+    """
+    fully_contained_index = strax.fully_contained_in(lone_hits, peaks)
+
+    for fc_i, lh_i in zip(fully_contained_index, lone_hits):
+        if fc_i == -1:
+            continue
+        p = peaks[fc_i]
+        lh_area = lh_i['area'] * to_pe[lh_i['channel']]
+        p['area'] += lh_area
+        p['area_per_channel'][lh_i['channel']] += lh_area
+
+        # Add lone hit as delta pulse to waveform:
+        index = (p['time'] - lh_i['time'])//p['dt']
+        p['data'][index] += lh_area
