@@ -10,7 +10,7 @@ from packaging import version
 import time
 import typing
 import warnings
-
+from enum import IntEnum
 import numpy as np
 
 import strax
@@ -84,6 +84,23 @@ class RunMetadataNotAvailable(Exception):
     pass
 
 
+@export
+class StorageType(IntEnum):
+    """
+    Class attribute of how far/close data is when fetched from a given
+    storage frontend. This is used to prioritize which frontend will be
+    asked first for data (prevents loading data from slow frontends when
+    fast frontends might also have the data)
+    """
+    # Feel free to add, could even be floats if needed
+    MEMORY = 0      # Sits in cache, super fast to load
+    LOCAL = 1       # Available on hard-disk, only limited by IO
+    ONLINE = 2      # Limited by network bandwidth
+    COMPRESSED = 3  # Data is compressed and needs (slow) decompression before loading
+    REMOTE = 4      # Data needs to be fetched from another host, and downloaded
+    TAPE = 10       # Nothing is as slow as tape, except pigeon post
+
+
 @export
 class StorageFrontend:
     """Interface to something that knows data-locations and run-level metadata.
@@ -92,6 +109,8 @@ class StorageFrontend:
     backends: list
     can_define_runs = False
     provide_run_metadata = False
+    provide_superruns = False
+    storage_type = StorageType.LOCAL
 
     def __init__(self,
                  readonly=False,
@@ -102,6 +121,8 @@ class StorageFrontend:
         """
         :param readonly: If True, throws CannotWriteData whenever saving is
         attempted.
+        :param provide_run_metadata: Boolean whether frontend can provide
+            run-level metadata.
         :param overwrite: When to overwrite data that already exists.
          - 'never': Never overwrite any data.
          - 'if_broken': Only overwrites data if it is incomplete or broken.
@@ -123,6 +144,7 @@ class StorageFrontend:
         self.overwrite = overwrite
         if provide_run_metadata is not None:
             self.provide_run_metadata = provide_run_metadata
+
         self.readonly = readonly
         self.log = logging.getLogger(self.__class__.__name__)
 
@@ -197,6 +219,15 @@ class StorageFrontend:
         return not (data_type in self.exclude
                     or self.take_only and data_type not in self.take_only)
 
+    def _support_superruns(self, run_id):
+        """Checks if run is a superrun and if superruns are provided by frontend"""
+        is_superrun = run_id.startswith(('_'))
+        if is_superrun:
+            return self.provide_superruns
+        else:
+            # Not a superrun
+            return True
+
     def find(self, key: DataKey,
              write=False,
              check_broken=True,
@@ -220,6 +251,9 @@ class StorageFrontend:
             raise DataNotAvailable(
                 f"{self} does not accept or provide data type {key.data_type}")
 
+        if not self._support_superruns(key.run_id):
+            raise DataNotAvailable(f'{self} does not support superruns: {key.run_id}.')
+
         if write:
             if self.readonly:
                 raise DataNotAvailable(f"{self} cannot write any-data, "
@@ -475,11 +509,17 @@ class StorageBackend:
                                     chunk_info=chunk_info,
                                     dtype=dtype,
                                     compressor=metadata['compressor'])
-
+        
+        _is_superrun = chunk_info['run_id'].startswith('_')
+        subruns=None
+        if _is_superrun:
+            subruns=chunk_info['subruns']
+            
         result = strax.Chunk(
             start=chunk_info['start'],
             end=chunk_info['end'],
             run_id=chunk_info['run_id'],
+            subruns=subruns,
             data=data,
             **chunk_construction_kwargs)
 
@@ -504,11 +544,32 @@ class StorageBackend:
             metadata['dtype'] = metadata['dtype'].descr.__repr__()
         return self._saver(key, metadata, **kwargs)
 
+    def get_metadata(self, backend_key: typing.Union[DataKey, str], **kwargs) -> dict:
+        """
+        Get the metadata using the backend_key and the Backend specific
+        _get_metadata method. When an unforeseen error occurs, raises an
+        strax.DataCorrupted error. Any kwargs are passed on to _get_metadata
+
+        :param backend_key: The key the backend should look for (can be string
+            or strax.DataKey)
+        :return: metadata for the data associated to the requested backend-key
+        :raises strax.DataCorrupted: This backend is not able to read the
+            metadata but it should exist
+        :raises strax.DataNotAvailable: When there is no data associated with
+            this backend-key
+        """
+        try:
+            return self._get_metadata(backend_key, **kwargs)
+        except (strax.DataCorrupted, strax.DataNotAvailable, NotImplementedError):
+            raise
+        except Exception as e:
+            raise strax.DataCorrupted(f'Cannot open metadata for {str(backend_key)}') from e
+
     ##
     # Abstract methods (to override in child)
     ##
 
-    def get_metadata(self, backend_key):
+    def _get_metadata(self, backend_key: typing.Union[DataKey, str], **kwargs) -> dict:
         """Return metadata of data described by key.
         """
         raise NotImplementedError
@@ -517,7 +578,7 @@ class StorageBackend:
         """Return a single data chunk"""
         raise NotImplementedError
 
-    def _saver(self, key, metadata):
+    def _saver(self, key, metadata, **kwargs):
         raise NotImplementedError
 
 
@@ -551,17 +612,32 @@ class Saver:
         pending = []
         exhausted = False
         chunk_i = 0
+
+        run_id = self.md['run_id']
+        _is_super_run = run_id.startswith('_')
         try:
             while not exhausted:
                 chunk = None
+
                 try:
                     if rechunk and self.allow_rechunk:
                         while (chunk is None or
                                 chunk.data.nbytes < chunk.target_size_mb*1e6):
-                            chunk = strax.Chunk.concatenate(
-                                [chunk, next(source)])
+                            next_chunk = next(source)
+
+                            if _is_super_run:
+                                # If we are creating a superrun, we load data from subruns
+                                # and the loaded subrun chunk becomes a superun chunk:
+                                next_chunk = strax.transform_chunk_to_superrun_chunk(run_id, 
+                                                                                     next_chunk)  
+                            chunk = strax.Chunk.concatenate([chunk, next_chunk])
                     else:
                         chunk = next(source)
+                        if _is_super_run:
+                            # If we are creating a superrun, we load data from subruns
+                            # and the loaded subrun chunk becomes a superun chunk:
+                            chunk = strax.transform_chunk_to_superrun_chunk(run_id, chunk)
+
                 except StopIteration:
                     exhausted = True
 
@@ -603,6 +679,7 @@ class Saver:
                           start=chunk.start,
                           end=chunk.end,
                           run_id=chunk.run_id,
+                          subruns=chunk.subruns,
                           nbytes=chunk.nbytes)
         if len(chunk) != 0 and 'time' in chunk.dtype.names:
             for desc, i in (('first', 0), ('last', -1)):
