@@ -3,7 +3,8 @@ import json
 import tempfile
 import os
 import os.path as osp
-
+import typing
+import time
 from bson import json_util
 import shutil
 
@@ -25,6 +26,7 @@ class DataDirectory(StorageFrontend):
 
     can_define_runs = True
     provide_run_metadata = False
+    provide_superruns = True
 
     def __init__(self, path='.', *args, deep_scan=False, **kwargs):
         """
@@ -62,6 +64,8 @@ class DataDirectory(StorageFrontend):
 
     def write_run_metadata(self, run_id, metadata):
         with open(self._run_meta_path(run_id), mode='w') as f:
+            if not 'name' in metadata:
+                metadata['name'] = run_id
             f.write(json.dumps(metadata, default=json_util.default))
 
     def _scan_runs(self, store_fields):
@@ -91,6 +95,7 @@ class DataDirectory(StorageFrontend):
 
     def _find(self, key, write,
               allow_incomplete, fuzzy_for, fuzzy_for_options):
+        self.raise_if_non_compatible_run_id(key.run_id)
         dirname = osp.join(self.path, str(key))
         exists = os.path.exists(dirname)
         bk = self.backend_key(dirname)
@@ -189,6 +194,11 @@ class DataDirectory(StorageFrontend):
         # (which FileStore should do) is sufficient.
         pass
 
+    @staticmethod
+    def raise_if_non_compatible_run_id(run_id):
+        if '-' in str(run_id):
+            raise ValueError("The filesystem frontend does not understand"
+                             " run_id's with '-', please replace with '_'")
 
 @export
 def dirname_to_prefix(dirname):
@@ -205,7 +215,7 @@ class FileSytemBackend(strax.StorageBackend):
     Metadata is stored in a file called metadata.json.
     """
 
-    def get_metadata(self, dirname):
+    def _get_metadata(self, dirname):
         prefix = dirname_to_prefix(dirname)
         metadata_json = f'{prefix}-metadata.json'
         md_path = osp.join(dirname, metadata_json)
@@ -261,6 +271,8 @@ class FileSytemBackend(strax.StorageBackend):
 class FileSaver(strax.Saver):
     """Saves data to compressed binary files"""
     json_options = dict(sort_keys=True, indent=4)
+    # When writing chunks, rewrite the json file every time we write a chunk
+    _flush_md_for_every_chunk = True
 
     def __init__(self, dirname, metadata, **kwargs):
         super().__init__(metadata, **kwargs)
@@ -325,11 +337,11 @@ class FileSaver(strax.Saver):
 
         if not self.is_forked or is_first:
             # Just append and flush the metadata
-            # (maybe not super-efficient to write the json everytime...
+            # (maybe not super-efficient to write the json every time...
             # just don't use thousands of chunks)
-            # TODO: maybe make option to turn this off?
             self.md['chunks'].append(chunk_info)
-            self._flush_metadata()
+            if self._flush_md_for_every_chunk:
+                self._flush_metadata()
 
     def _close(self):
         if not os.path.exists(self.tempdirname):
@@ -353,3 +365,111 @@ class FileSaver(strax.Saver):
 @export
 class InvalidFolderNameFormat(Exception):
     pass
+
+
+@export
+def rechunker(source_directory:str,
+              dest_directory: typing.Optional[str] = None,
+              replace: bool = False,
+              compressor: typing.Optional[str] = None,
+              target_size_mb: typing.Optional[str] = None,
+              rechunk: bool = True,
+              )-> dict:
+    """
+    Rechunk/Recompress a strax-datatype saved in a FileSystemBackend
+    outside of a strax.Context. For a user-friendly context centered
+    alternative, see strax.Context.copy_to_frontend:
+    github.com/AxFoundation/strax/blob/a0d51fdd3bea52c228c8f74c614fc77bb7cf1bc5/strax/context.py#L1792  # noqa
+
+    One can either specify a destination directory where to store a new
+    copy of this data with <dest_directory> or replace the input file
+    with it's rechunked version.
+
+    This function can either:
+     - rechunk (if <rechunk? is True), probably incrementing the
+        <target_size_mb> is also useful (create larger chunks)
+     - recompress (if a <compressor> is specified)
+
+    One can also rechunk and recompress simultaneously
+
+    :param source_directory: Path to a folder containing a single
+        strax.DataType.
+    :param dest_directory: Head of a folder whereto write new data. If
+        nothing is specified, write to a temporary directory.
+    :param replace: Delete the source_directory and replace it with it's
+        rechunked version
+    :param compressor: Compressor to be used in saving the rechunked
+        data
+    :param target_size_mb: Target size of chunks (uncompressed). As long
+        as a chunk is smaller than this many MB, keep adding new MBs
+        until the chunk is at least target_size_mb or we run out of
+        chunks.
+    :param rechunk: Do we want to rechunk?
+    :return: Dictionary with some information on the write/load times
+        involved.
+    """
+    if not os.path.exists(source_directory):
+        raise FileNotFoundError(f'No file at {source_directory}')
+    if not replace and dest_directory is None:
+        raise ValueError(f'Specify a destination path <dest_file> when '
+                         f'not replacing the original path')
+    backend_key = os.path.basename(os.path.normpath(source_directory))
+
+    if dest_directory is None and replace:
+        _temp_dir = tempfile.TemporaryDirectory()
+        dest_directory = _temp_dir.name
+    else:
+        _temp_dir = False
+
+
+    if os.path.basename(os.path.normpath(dest_directory)) != backend_key:
+        # New key should be correct! If there is not an exact match,
+        # we want to make sure that we append the backend_key correctly
+        print(f'Will write to {dest_directory} and make sub-folder {backend_key}')
+        dest_directory = os.path.join(dest_directory, backend_key)
+    backend = strax.FileSytemBackend()
+    meta_data = backend.get_metadata(source_directory)
+    source_compressor = meta_data['compressor']
+
+    if compressor is not None:
+        meta_data['compressor'] = compressor
+    if target_size_mb is not None:
+        meta_data['chunk_target_size_mb'] = target_size_mb
+
+    data_loader = backend.loader(source_directory)
+
+    load_time_seconds = []
+    def loader():
+        """Wrapped loader for bookkeeping load time"""
+        while True:
+            try:
+                t0 = time.time()
+                data = next(data_loader)
+                # Update target chunk size for re-chunking
+                data.target_size_mb = meta_data['chunk_target_size_mb']
+                load_time_seconds.append(time.time()-t0)
+            except StopIteration:
+                return
+            yield data
+    print(f'Rechunking {source_directory} to {dest_directory}')
+    saver = backend._saver(dest_directory, metadata=meta_data)
+
+    write_time_start = time.time()
+    saver.save_from(loader(), rechunk=rechunk)
+    load_time = sum(load_time_seconds)
+    write_time = time.time() - write_time_start - load_time
+
+    if replace:
+        print(f'move {dest_directory} to {source_directory}')
+        shutil.rmtree(source_directory)
+        shutil.move(dest_directory, source_directory)
+    if _temp_dir:
+        _temp_dir.cleanup()
+
+    return dict(backend_key=backend_key,
+                load_time=load_time,
+                write_time=write_time,
+                uncompressed_mb= sum([x['nbytes'] for x in meta_data['chunks']]) / 1e6,
+                source_compressor=source_compressor,
+                dest_compressor=meta_data['compressor'],
+                )
