@@ -52,7 +52,6 @@ def sorted_bounds(disjoint=False,
 # Fake intervals
 ##
 
-# TODO: isn't this duplicated with bounds_to_records??
 
 def bounds_to_intervals(bs, dtype=strax.interval_dtype):
     x = np.zeros(len(bs), dtype=dtype)
@@ -139,8 +138,11 @@ several_fake_records_one_channel = sorted_bounds(
 # Basic test plugins
 ##
 @strax.takes_config(
-    strax.Option('crash', default=False),
-    strax.Option('secret_time_offset', default=0, track=False)
+    strax.Option('crash', type=bool, default=False),
+    strax.Option('dummy_tracked_option', type=int, default=42),
+    strax.Option('secret_time_offset', type=int, default=0, track=False),
+    strax.Option('n_chunks', type=int, default=10, track=False),
+    strax.Option('recs_per_chunk', type=int, default=10, track=False),
 )
 class Records(strax.Plugin):
     provides = 'records'
@@ -154,12 +156,12 @@ class Records(strax.Plugin):
         return True
 
     def is_ready(self, chunk_i):
-        return chunk_i < n_chunks
+        return chunk_i < self.config['n_chunks']
 
     def compute(self, chunk_i):
         if self.config['crash']:
             raise SomeCrash("CRASH!!!!")
-        r = np.zeros(recs_per_chunk, self.dtype)
+        r = np.zeros(self.config['recs_per_chunk'], self.dtype)
         t0 = chunk_i + self.config['secret_time_offset']
         r['time'] = t0
         r['length'] = r['dt'] = 1
@@ -172,8 +174,8 @@ class SomeCrash(Exception):
 
 
 @strax.takes_config(
-    strax.Option('base_area', default=0),
-    strax.Option('give_wrong_dtype', default=False),
+    strax.Option('base_area', type=int, default=0),
+    strax.Option('give_wrong_dtype', type=bool, default=False),
     strax.Option('bonus_area', default_by_run=[(0, 0), (1, 1)]))
 class Peaks(strax.Plugin):
     provides = 'peaks'
@@ -184,7 +186,7 @@ class Peaks(strax.Plugin):
 
     def compute(self, records):
         if self.config['give_wrong_dtype']:
-            return np.zeros(5, [('a', np.int), ('b', np.float)])
+            return np.zeros(5, [('a', np.int64), ('b', np.float64)])
         p = np.zeros(len(records), self.dtype)
         p['time'] = records['time']
         p['length'] = p['dt'] = 1
@@ -195,25 +197,34 @@ class Peaks(strax.Plugin):
 # Another peak-kind plugin, to test time_range selection
 # with unaligned chunks
 class PeakClassification(strax.Plugin):
-    provides = 'peak_classification'
-    data_kind = 'peaks'
+    provides = ('peak_classification',
+                'lone_hits')
+    data_kind = dict(peak_classification='peaks',
+                     lone_hits='lone_hits')
     depends_on = ('peaks',)
-    dtype = (
-        [('type', np.int8, 'Classification of the peak.')]
-        + strax.time_fields)
     rechunk_on_save = True
+    save_when = immutabledict({k: strax.SaveWhen.ALWAYS for k in provides})
+
+    def infer_dtype(self):
+        peaks_dtype = strax.time_fields + [('type', np.int8, 'Classification of the peak.')]
+        lone_hits = strax.time_fields
+        return {'peak_classification': peaks_dtype,
+                'lone_hits': lone_hits}
 
     def compute(self, peaks):
-        return dict(type=np.zeros(len(peaks)),
-                    time=peaks['time'],
-                    endtime=strax.endtime(peaks))
+        p = np.zeros(len(peaks), self.dtype['peak_classification'])
+        p['time'] = peaks['time']
+        p['endtime'] = strax.endtime(peaks)
 
+        lh = np.zeros(len(peaks), self.dtype['lone_hits'])
+        lh['time'] = peaks['time']
+        lh['endtime'] = strax.endtime(peaks)
+        return dict(peak_classification=p,
+                    lone_hits=lh)
 
-recs_per_chunk = 10
-n_chunks = 10
+# Used in test_core.py
 run_id = '0'
 
-
 ##
 # Some test plugins to check
 # inheritance of "child"-plugins.
@@ -230,7 +241,9 @@ DEFAULT_CONFIG_TEST = {'area_parent': 2, 'area_child': 4,
 
 # Parent:
 @strax.takes_config(
-    strax.Option('by_child_overwrite_option', default=DEFAULT_CONFIG_TEST['area_parent'],
+    strax.Option('by_child_overwrite_option',
+                 type=int,
+                 default=DEFAULT_CONFIG_TEST['area_parent'],
                  help="Option we will overwrite in our child plugin"),
     strax.Option('parent_unique_option', type=int, default=DEFAULT_CONFIG_TEST['max_gap_both'],
                  help='Option which is not touched by the child and '
@@ -272,6 +285,7 @@ class ParentPlugin(strax.Plugin):
 # Child:
 @strax.takes_config(
     strax.Option('by_child_overwrite_option_child',
+                 type=int,
                  default=DEFAULT_CONFIG_TEST['area_child'],
                  child_option=True,
                  parent_option_name='by_child_overwrite_option',
@@ -284,10 +298,11 @@ class ParentPlugin(strax.Plugin):
     strax.Option('child_exclusive_option', type=int, default=DEFAULT_CONFIG_TEST['nhits_child'],
                  help='Option which is exclusive for the child.'),
     strax.Option('more_special_context_option_child',
+                 type=immutabledict,
                  child_option=True,
                  parent_option_name='more_special_context_option',
                  track=False,
-                 default=immutabledict(tpc=DEFAULT_CONFIG_TEST['channel_map_child']), type=immutabledict,
+                 default=immutabledict(tpc=DEFAULT_CONFIG_TEST['channel_map_child']),
                  help="Special context option which is not tacked e.g. channel_map")
 )
 class ChildPlugin(ParentPlugin):
