@@ -13,20 +13,26 @@ def test_core():
         for max_workers in [1, 2]:
             mystrax = strax.Context(storage=[],
                                     register=[Records, Peaks],
-                                    allow_multiprocess=allow_multiprocess)
+                                    allow_multiprocess=allow_multiprocess,
+                                    use_per_run_defaults=True,
+                                    )
             bla = mystrax.get_array(run_id=run_id, targets='peaks',
                                     max_workers=max_workers)
-            assert len(bla) == recs_per_chunk * n_chunks
+            p = mystrax.get_single_plugin(run_id, 'records')
+            assert len(bla) == p.config['recs_per_chunk'] * p.config['n_chunks']
             assert bla.dtype == strax.peak_dtype()
 
 
 def test_multirun():
     for max_workers in [1, 2]:
         mystrax = strax.Context(storage=[],
-                                register=[Records, Peaks],)
+                                register=[Records, Peaks],
+                                use_per_run_defaults=True,
+                                )
         bla = mystrax.get_array(run_id=['0', '1'], targets='peaks',
                                 max_workers=max_workers)
-        n = recs_per_chunk * n_chunks
+        p = mystrax.get_single_plugin(run_id, 'records')
+        n = p.config['recs_per_chunk'] * p.config['n_chunks']
         assert len(bla) == n * 2
         np.testing.assert_equal(
             bla['run_id'],
@@ -37,7 +43,9 @@ def test_filestore():
     with tempfile.TemporaryDirectory() as temp_dir:
         mystrax = strax.Context(storage=strax.DataDirectory(temp_dir,
                                                             deep_scan=True),
-                                register=[Records, Peaks])
+                                register=[Records, Peaks],
+                                use_per_run_defaults=True,
+                                )
 
         assert not mystrax.is_stored(run_id, 'peaks')
         mystrax.scan_runs()
@@ -45,7 +53,8 @@ def test_filestore():
 
         # Create it
         peaks_1 = mystrax.get_array(run_id=run_id, targets='peaks')
-        assert len(peaks_1) == recs_per_chunk * n_chunks
+        p = mystrax.get_single_plugin(run_id, 'records')
+        assert len(peaks_1) == p.config['recs_per_chunk'] * p.config['n_chunks']
 
         assert mystrax.is_stored(run_id, 'peaks')
         mystrax.scan_runs()
@@ -83,6 +92,7 @@ def test_filestore():
         assert osp.exists(zf)
 
         mystrax = strax.Context(storage=strax.ZipDirectory(temp_dir),
+                                use_per_run_defaults=True,
                                 register=[Records, Peaks])
         metadata_2 = mystrax.get_meta(run_id, 'peaks')
         assert metadata == metadata_2
@@ -98,7 +108,9 @@ def test_datadirectory_deleted():
 
         mystrax = strax.Context(storage=strax.DataDirectory(data_dir,
                                                             deep_scan=True),
-                                register=[Records, Peaks])
+                                register=[Records, Peaks],
+                                use_per_run_defaults=True,
+                                )
 
         # Delete directory AFTER context is created
         shutil.rmtree(data_dir)
@@ -118,7 +130,9 @@ def test_fuzzy_matching():
     with tempfile.TemporaryDirectory() as temp_dir:
         st = strax.Context(storage=strax.DataDirectory(temp_dir,
                                                        deep_scan=True),
-                           register=[Records, Peaks])
+                           register=[Records, Peaks],
+                           use_per_run_defaults=True,
+                           )
 
         st.make(run_id=run_id, targets='peaks')
 
@@ -144,6 +158,7 @@ def test_fuzzy_matching():
     with tempfile.TemporaryDirectory() as temp_dir:
         st = strax.Context(storage=strax.DataDirectory(temp_dir),
                            register=[Records, Peaks],
+                           use_per_run_defaults=True,
                            fuzzy_for=('records',))
         st.make(run_id, 'peaks')
         assert not st.is_stored(run_id, 'peaks')
@@ -153,7 +168,9 @@ def test_fuzzy_matching():
 def test_storage_converter():
     with tempfile.TemporaryDirectory() as temp_dir:
         st = strax.Context(storage=strax.DataDirectory(temp_dir),
-                           register=[Records, Peaks])
+                           register=[Records, Peaks],
+                           use_per_run_defaults=True,
+                           )
         st.make(run_id=run_id, targets='peaks')
 
         with tempfile.TemporaryDirectory() as temp_dir_2:
@@ -162,6 +179,7 @@ def test_storage_converter():
                                              readonly=True),
                          strax.DataDirectory(temp_dir_2)],
                 register=[Records, Peaks],
+                use_per_run_defaults=True,
                 storage_converter=True)
             store_1, store_2 = st.storage
 
@@ -184,7 +202,9 @@ def test_exception():
             st = strax.Context(storage=strax.DataDirectory(temp_dir),
                                register=[Records, Peaks],
                                allow_multiprocess=allow_multiprocess,
-                               config=dict(crash=True))
+                               config=dict(crash=True),
+                               use_per_run_defaults=True,
+                               )
 
             # Check correct exception is thrown
             with pytest.raises(SomeCrash):
@@ -211,7 +231,9 @@ def test_exception_in_saver(caplog):
 
     with tempfile.TemporaryDirectory() as temp_dir:
         st = strax.Context(storage=strax.DataDirectory(temp_dir),
-                           register=[Records, Peaks])
+                           register=[Records, Peaks],
+                           use_per_run_defaults=True,
+                           )
 
         def kaboom(*args, **kwargs):
             raise SomeCrash
@@ -234,7 +256,9 @@ def test_random_access():
         Peaks.rechunk_on_save = False
 
         st = strax.Context(storage=strax.DataDirectory(temp_dir),
-                           register=[Records, Peaks, PeakClassification])
+                           register=[Records, Peaks, PeakClassification],
+                           use_per_run_defaults=True,
+                           )
 
         with pytest.raises(strax.DataNotAvailable):
             # Time range selection requires data already available
@@ -254,7 +278,8 @@ def test_random_access():
         df = st.get_array(run_id, 'peaks', time_range=(3, 5))
         # Also test without the progress-bar
         df_pbar = st.get_array(run_id, 'peaks', time_range=(3, 5), progress_bar = False)
-        assert len(df) == 2 * recs_per_chunk
+        p = st.get_single_plugin(run_id, 'records')
+        assert len(df) == 2 * p.config['recs_per_chunk']
         assert df['time'].min() == 3
         assert df['time'].max() == 4
         assert np.all(df == df_pbar), 'progress-bar changes the result?!?'
@@ -262,7 +287,7 @@ def test_random_access():
         # Try again with unaligned chunks
         df = st.get_array(run_id, ['peaks', 'peak_classification'],
                           time_range=(3, 5))
-        assert len(df) == 2 * recs_per_chunk
+        assert len(df) == 2 * p.config['recs_per_chunk']
         assert df['time'].min() == 3
         assert df['time'].max() == 4
 
@@ -281,7 +306,9 @@ def test_rechunk_on_save():
 
         with tempfile.TemporaryDirectory() as temp_dir:
             st = strax.Context(storage=strax.DataDirectory(path=temp_dir),
-                               register=[Records, Peaks])
+                               register=[Records, Peaks],
+                               use_per_run_defaults=True,
+                               )
 
             peaks_0 = st.get_array('0', 'peaks')
             peaks_0a = st.get_array('0', 'peaks')
@@ -298,7 +325,6 @@ def test_run_selection():
     with tempfile.TemporaryDirectory() as temp_dir:
         sf = strax.DataDirectory(path=temp_dir,
                                  deep_scan=True, provide_run_metadata=True)
-
         # Write mock runs db
         for d in mock_rundb:
             sf.write_run_metadata(d['name'], d)
@@ -330,7 +356,8 @@ def test_run_defaults():
         for d in mock_rundb:
             sf.write_run_metadata(d['name'], d)
         st = strax.Context(storage=sf, register=[Records, Peaks],
-                           use_per_run_defaults=True)
+                           use_per_run_defaults=True,
+                           )
 
         # The run defaults get used
         peaks = st.get_array('0', 'peaks')
@@ -344,6 +371,7 @@ def test_run_defaults():
 def test_dtype_mismatch():
     mystrax = strax.Context(storage=[],
                             register=[Records, Peaks],
+                            use_per_run_defaults=True,
                             config=dict(give_wrong_dtype=True))
     with pytest.raises(strax.PluginGaveWrongOutput):
         mystrax.get_array(run_id=run_id, targets='peaks')
@@ -351,67 +379,23 @@ def test_dtype_mismatch():
 
 def test_get_single_plugin():
     mystrax = strax.Context(storage=[],
-                            register=[Records, Peaks])
+                            register=[Records, Peaks],
+                            use_per_run_defaults=True,
+                            )
     p = mystrax.get_single_plugin('0', 'peaks')
     assert isinstance(p, Peaks)
     assert len(p.config)
     assert p.config['base_area'] == 0
 
 
-def test_superrun():
-    # TODO: duplicated init with test_run_selection
-    mock_rundb = [
-        dict(name='0', start=0, end=int(1e9)),
-        dict(name='1', start=int(2e9), end=int(3e9))
-    ]
-
-    with tempfile.TemporaryDirectory() as temp_dir:
-        # Test run definition
-        sf = strax.DataDirectory(path=temp_dir,
-                                 provide_run_metadata=True,
-                                 deep_scan=True)
-        for d in mock_rundb:
-            sf.write_run_metadata(d['name'], d)
-
-        st = strax.Context(storage=sf, register=[Records, Peaks])
-        st.define_run('super', ['0', '1'])
-
-        md = st.run_metadata('_super')
-        assert md['start'] == 0
-        assert md['end'] == int(3e9)
-        assert md['livetime'] == int(2e9)
-        assert md['sub_run_spec'] == {'0': 'all', '1': 'all'}
-
-        # Test superrun loading
-        r1 = st.get_array('0', 'records')
-        r2 = st.get_array('1', 'records', config=dict(secret_time_offset=int(1e9)))
-        rs = st.get_array('_super', 'records')
-        np.testing.assert_array_equal(rs, np.concatenate([r1, r2]))
-
-        # Test that superrun loading triggers subrun processing
-        ps = st.get_array('_super', 'peaks')
-        p1 = st.get_array('0', 'peaks')
-        p2 = st.get_array('1', 'peaks')
-        np.testing.assert_array_equal(p1['area'], np.zeros(len(p1)))
-        np.testing.assert_array_equal(p2['area'], np.ones(len(p2)))
-        np.testing.assert_array_equal(ps, np.concatenate([p1, p2]))
-
-        # Test superrun processing can override run-dependent options
-        st.set_config(config=dict(bonus_area=0))
-        ps = st.get_array('_super', 'peaks')
-        p1 = st.get_array('0', 'peaks')
-        p2 = st.get_array('1', 'peaks')
-        np.testing.assert_array_equal(p1['area'], np.zeros(len(p1)))
-        np.testing.assert_array_equal(p2['area'], np.zeros(len(p2)))
-        np.testing.assert_array_equal(ps, np.concatenate([p1, p2]))
-
-
 def test_allow_multiple(targets=('peaks', 'records')):
     """Test if we can use the allow_multiple correctly and fail otherwise"""
     with tempfile.TemporaryDirectory() as temp_dir:
         mystrax = strax.Context(storage=strax.DataDirectory(temp_dir,
                                                             deep_scan=True),
-                                register=[Records, Peaks])
+                                register=[Records, Peaks],
+                                use_per_run_defaults=True,
+                                )
         mystrax.set_context_config({'allow_lazy': False,
                                     'timeout': 80})
 
@@ -448,3 +432,22 @@ def test_allow_multiple_inverted():
     # actually depending on the second. In that case, we should
     # subscribe the first target as the endpoint of the processing
     test_allow_multiple(targets=('records', 'peaks',))
+
+
+def test_available_for_run():
+    """Very simply test the available_for_run function"""
+    with tempfile.TemporaryDirectory() as temp_dir:
+        mystrax = strax.Context(storage=strax.DataDirectory(temp_dir,
+                                                            deep_scan=True),
+                                register=[Records, Peaks],
+                                use_per_run_defaults=True,
+                                )
+        targets = list(mystrax._plugin_class_registry.keys())
+        for exclude_i in range(len(targets)):
+            for include_i in range(len(targets)):
+                df = mystrax.available_for_run(run_id,
+                                               include_targets = targets[:include_i],
+                                               exclude_targets = targets[:exclude_i])
+                if len(df):
+                    # We haven't made any data
+                    assert not sum(df['is_stored'])
